// 
// example of Monte Carlo simulation method
//
// Time 1400s on Athlon64/3000+

#include <map>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <cmath>
#include <cstdlib>
#include <ctime>

using namespace std;

const long NEXP = 1000000000;	// max number of experiments

// generate random number <0..1)
inline double Random()
{
    return rand() / (RAND_MAX + 1.0);   // rand() can return RAND_MAX
}

int main()
{
    map<long,double> maxerr;
    map<long,double>::iterator it;
    ofstream out("monte-carlo-2.dat");
    srand(time(0));		// randomize 
    clock_t start = clock();
    for (int j = 0; j <= 10; j++) {
        long S = 0;
        long N = 100;		// for print only
	for (int i = 0; i <= NEXP; i++) { // experiments
	    double x = Random();
	    double y = Random();
	    if ((x * x + y * y) < 1.0)
		S++;
	    if (i == N) {	// print results
		double pi = 4.0 * (double) S / i;
                double relerr = fabs((pi - M_PI) / M_PI * 100);
		if(relerr > maxerr[i])
                    maxerr[i] = relerr;
		N *= 10;
	    }
	}
    }
    for(it = maxerr.begin(); it != maxerr.end(); it++) 
        out << it->first << "\t" << it->second << endl;
    cout << "Time: ";
    cout << setprecision(4) << double(clock()-start)/CLOCKS_PER_SEC;
    cout << " s" << endl;
}
