// 
// example of Monte Carlo simulation method
// integral of function sin(x)
//
// $r = \int_0^\pi sin(x) dx$
// Analytical solution: r = 2
//

#include <iostream>
#include <iomanip>
#include <cstdlib>
#include <cmath>
#include <ctime>
using namespace std;

const long NEXP = 10000000;	// max number of experiments

// generate random number <0..1)
inline double Random()
{
    return rand() / (RAND_MAX + 1.0);   // rand() can return RAND_MAX
}

int main()
{
    srand(time(0));
    long N = 100;		// for print only
    long S = 0;
    clock_t start = clock();
    double sum=0;
    cout << endl;
    for (int i = 0; i <= NEXP; i++) {
#ifdef METHOD1
	double x = Random()*M_PI;
	double y = Random();
	if (y < sin(x))
	    S++;
	if (i == N) {		// print results
	    double probability = (double) S / i;
	    double result = M_PI * probability;
	    cout << "result = " << result << "\t";
	    cout << "(" << i << " experiments)" <<  endl;
	    N *= 10;
	}
#else // METHOD2
	double x = Random()*M_PI;
        sum += sin(x);
	if (i == N) {		// print results
	    double average = sum/i;
	    double result = M_PI * average;
	    cout << "result = " << result << "\t";
	    cout << "(" << i << " experiments)" <<  endl;
	    N *= 10;
	}
#endif
    }
    cout << endl;
    cout << "Exact solution: " << 2.0 << endl;
    cout << "Time: ";
    cout << setprecision(4) << double(clock()-start)/CLOCKS_PER_SEC;
    cout << " s" << endl;
}

